import logging
import os
import shutil

import rifle.checker as checker
import rifle.utils as utils
from rifle.errors import ValidationError


def main():
    '''
        Entry point for rifle.
    '''
    # Set up the parser
    parser = utils.rifle_cmdline_args()
    args = parser.parse_args()

    # Set up the logger
    utils.setup_logging(args.verbose)
    logger = logging.getLogger()
    logger.handlers = []
    ch = logging.StreamHandler()
    ch.setFormatter(utils.ColoredFormatter())
    logger.addHandler(ch)
    fh = logging.FileHandler('run.log', 'w')
    logger.addHandler(fh)

    work_dir = os.path.join(os.getcwd(), args.work_dir)
    if not os.path.exists(work_dir):
        logger.debug('Creating new work directory: ' + work_dir)
        os.mkdir(work_dir)
    else:
        logger.debug('Removing old work directory: ' + work_dir)
        shutil.rmtree(work_dir)
        logger.debug('Creating new work directory: ' + work_dir)
        os.mkdir(work_dir)

    try:
        checker.check_specs(os.path.abspath(args.isa_spec),
                            os.path.abspath(args.platform_spec), work_dir)
    except ValidationError as msg:
        logger.error(msg)
        return 1


if __name__ == "__main__":
    exit(main())



import os
import logging

from cerberus import Validator

import rifle.utils as utils
from rifle.errors import ValidationError
from rifle.schemaValidator import schemaValidator
import rifle.constants as constants
from rifle.utils import yaml

logger = logging.getLogger(__name__)


def iset():
    '''Function to check and set defaults for all "implemented" fields which are dependent on 
        the xlen.'''
    global inp_yaml
    if '32' in inp_yaml['ISA']:
        return False
    else:
        return True


def nosset():
    '''Function to check and set defaults for all fields which are dependent on 
        the presence of 'S' extension and have a hardwired value of 0.'''
    global inp_yaml
    if 'S' not in inp_yaml['ISA']:
        return {'is_hardwired': True, 'hardwired_val': 0}
    else:
        return {'is_hardwired': False}


def nouset():
    '''Function to check and set defaults for all fields which are dependent on 
        the presence of 'U' extension and have a hardwired value of 0.'''
    global inp_yaml
    if 'U' not in inp_yaml['ISA']:
        return {'is_hardwired': True, 'hardwired_val': 0}
    else:
        return {'is_hardwired': False}


def upieset(doc):
    '''Function to check and set value for upie field in misa.'''
    global inp_yaml
    if 'U' not in inp_yaml['ISA']:
        return {'is_hardwired': True, 'hardwired_val': 0}
    elif 'UPIE' not in doc.keys():
        return {'is_hardwired': False}
    else:
        return doc['UPIE']


def uieset(doc):
    '''Function to check and set value for uie field in misa.'''
    global inp_yaml
    if 'U' not in inp_yaml['ISA']:
        return {'is_hardwired': True, 'hardwired_val': 0}
    elif 'UIE' not in doc.keys():
        return {'is_hardwired': False}
    else:
        return doc['UIE']


def twset():
    '''Function to check and set value for tw field in misa.'''
    global inp_yaml
    if 'S' not in inp_yaml['ISA'] and 'U' not in inp_yaml['ISA']:
        return {'is_hardwired': True, 'hardwired_val': 0}
    else:
        return {'is_hardwired': False}


def miedelegset():
    '''Function to set "implemented" value for mideleg regisrer.'''
    # return True
    global inp_yaml
    if 'U' not in inp_yaml['ISA']:
        return False
    elif (('U' in inp_yaml['ISA']) and
          not ('N' in inp_yaml['ISA'] or 'S' in inp_yaml['ISA'])):
        return False
    else:
        return True


def mepcset():
    return {
        'range': {
            'rangelist': [[0, int("FFFFFFFF", 16)]],
            'mode': "Unchanged"
        }
    }


def xtvecset():
    return {
        'BASE': {
            'range': {
                'rangelist': [[0, int("3FFFFFFF", 16)]],
                'mode': "Unchanged"
            }
        },
        'MODE': {
            'range': {
                'rangelist': [[0]],
                'mode': "Unchanged"
            }
        }
    }


def simpset():
    global inp_yaml
    if 'S' in inp_yaml['ISA']:
        return True
    else:
        return False


def satpset():
    return {'MODE': {'range': {'rangelist': [[0]]}}}


def findxlen():
    global inp_yaml
    if "32" in inp_yaml['ISA']:
        xlen = 32
    elif "64" in inp_yaml['ISA']:
        xlen = 64
    elif "128" in inp_yaml['ISA']:
        xlen = 128
    return xlen


def xlenset():
    return findxlen()


def add_def_setters(schema_yaml):
    '''Function to set the default setters for various fields in the schema'''
    schema_yaml['mstatus']['schema']['SXL']['schema']['implemented'][
        'default_setter'] = lambda doc: iset()
    schema_yaml['mstatus']['schema']['UXL']['schema']['implemented'][
        'default_setter'] = lambda doc: iset()
    schema_yaml['mstatus']['schema']['TVM'][
        'default_setter'] = lambda doc: nosset()
    schema_yaml['mstatus']['schema']['TSR'][
        'default_setter'] = lambda doc: nosset()
    schema_yaml['mstatus']['schema']['MXR'][
        'default_setter'] = lambda doc: nosset()
    schema_yaml['mstatus']['schema']['SUM'][
        'default_setter'] = lambda doc: nosset()
    schema_yaml['mstatus']['schema']['SPP'][
        'default_setter'] = lambda doc: nosset()
    schema_yaml['mstatus']['schema']['SPIE'][
        'default_setter'] = lambda doc: nosset()
    schema_yaml['mstatus']['schema']['SIE'][
        'default_setter'] = lambda doc: nosset()
    schema_yaml['mstatus']['schema']['UPIE'][
        'default_setter'] = lambda doc: upieset(doc)
    schema_yaml['mstatus']['schema']['UIE'][
        'default_setter'] = lambda doc: uieset(doc)
    schema_yaml['mstatus']['schema']['MPRV'][
        'default_setter'] = lambda doc: nouset()
    schema_yaml['mstatus']['schema']['TW'][
        'default_setter'] = lambda doc: twset()
    schema_yaml['mideleg']['schema']['implemented'][
        'default_setter'] = lambda doc: miedelegset()
    schema_yaml['medeleg']['schema']['implemented'][
        'default_setter'] = lambda doc: miedelegset()
    schema_yaml['mepc']['default_setter'] = lambda doc: mepcset()
    schema_yaml['mtvec']['default_setter'] = lambda doc: xtvecset()
    schema_yaml['stvec']['default_setter'] = lambda doc: xtvecset()
    schema_yaml['satp']['default_setter'] = lambda doc: satpset()
    schema_yaml['stvec']['schema']['implemented'][
        'default_setter'] = lambda doc: simpset()
    schema_yaml['sie']['schema']['implemented'][
        'default_setter'] = lambda doc: simpset()
    schema_yaml['sip']['schema']['implemented'][
        'default_setter'] = lambda doc: simpset()
    schema_yaml['scounteren']['schema']['implemented'][
        'default_setter'] = lambda doc: simpset()
    schema_yaml['sepc']['schema']['implemented'][
        'default_setter'] = lambda doc: simpset()
    schema_yaml['satp']['schema']['implemented'][
        'default_setter'] = lambda doc: simpset()
    schema_yaml['xlen']['default_setter'] = lambda doc: xlenset()
    return schema_yaml


def imp_normalise(foo):
    '''
        Function to trim the dictionary. Any node with implemented field set to false is trimmed of all the other nodes.
        
        :param foo: The dictionary to be trimmed.
        
        :type foo: dict

        :return: The trimmed dictionary.
    '''
    for key in foo.keys():
        if isinstance(foo[key], dict):
            foo[key] = imp_normalise(foo[key])
        if key == 'implemented':
            if not foo[key]:
                foo = {'implemented': False}
                break
    return foo


def errPrint(foo, space=' '):
    '''
        Function to petty print the error from cerberus.
    '''
    error = ''
    for key in foo.keys():
        error += space + str(key) + ":"
        if isinstance(foo[key][0], dict):
            error += "\n" + errPrint(foo[key][0], space + space)
        else:
            error += str(foo[key][0])
        error += "\n"
    return error.rstrip()


def check_specs(isa_spec, platform_spec, work_dir):
    ''' 
        Function to perform ensure that the isa and platform specifications confirm
        to their schemas. The :py:mod:`Cerberus` module is used to validate that the
        specifications confirm to their respective schemas.

        :param isa_spec: The path to the DUT isa specification yaml file. 

        :param platform_spec: The path to the DUT platform specification yaml file.
        
        :type isa_spec: str

        :type platform_spec: str

        :raise ValidationError: It is raised when the specifications violate the 
            schema rules.
        
        :return: A tuple with the first entry being the path to normalized isa file 
            and the second being path to the platform spec file.
    '''
    global inp_yaml

    logger.info('Input-ISA file')

    foo = isa_spec
    schema = constants.isa_schema
    """
      Read the input-isa foo (yaml file) and validate with schema-isa for feature values
      and constraints
    """
    # Load input YAML file
    logger.info('Loading input file: ' + str(foo))
    inp_yaml = utils.load_yaml(foo)

    # instantiate validator
    logger.info('Load Schema ' + str(schema))
    schema_yaml = utils.load_yaml(schema)

    #Extract xlen
    xlen = findxlen()

    schema_yaml = add_def_setters(schema_yaml)
    validator = schemaValidator(schema_yaml, xlen=xlen)
    validator.allow_unknown = False
    validator.purge_readonly = True
    normalized = validator.normalized(inp_yaml, schema_yaml)

    # Perform Validation
    logger.info('Initiating Validation')
    valid = validator.validate(inp_yaml)

    # Print out errors
    if valid:
        logger.info('No Syntax errors in Input ISA Yaml. :)')
    else:
        error_list = validator.errors
        logger.error("Error in " + foo + "\n" + errPrint(error_list))
        raise ValidationError(
            "Error in ISA Yaml. Refer to logs for more details.")

    file_name = os.path.split(foo)
    file_name_split = file_name[1].split('.')
    output_filename = os.path.join(
        work_dir, file_name_split[0] + '_checked.' + file_name_split[1])
    ifile = output_filename
    outfile = open(output_filename, 'w')
    logger.info('Dumping out Normalized Checked YAML: ' + output_filename)
    yaml.dump(imp_normalise(normalized), outfile)

    logger.info('Input-Platform file')

    foo = platform_spec
    schema = constants.platform_schema
    """
      Read the input-platform foo (yaml file) and validate with schema-platform for feature values
      and constraints
    """
    # Load input YAML file
    logger.info('Loading input file: ' + str(foo))
    inp_yaml = utils.load_yaml(foo)
    if inp_yaml is None:
        inp_yaml = {'mtime': {'implemented': False}}

    # instantiate validator
    logger.info('Load Schema ' + str(schema))
    schema_yaml = utils.load_yaml(schema)

    validator = schemaValidator(schema_yaml, xlen=xlen)
    validator.allow_unknown = False
    validator.purge_readonly = True
    normalized = validator.normalized(inp_yaml, schema_yaml)

    # Perform Validation
    logger.info('Initiating Validation')
    valid = validator.validate(inp_yaml)

    # Print out errors
    if valid:
        logger.info('No Syntax errors in Input Platform Yaml. :)')
    else:
        error_list = validator.errors
        logger.error("Error in " + foo + "\n" + errPrint(error_list))
        raise ValidationError("Error in Platform\
             Yaml. Refer to logs for more details.")

    file_name = os.path.split(foo)
    file_name_split = file_name[1].split('.')
    output_filename = os.path.join(
        work_dir, file_name_split[0] + '_checked.' + file_name_split[1])
    pfile = output_filename
    outfile = open(output_filename, 'w')
    logger.info('Dumping out Normalized Checked YAML: ' + output_filename)
    yaml.dump(imp_normalise(normalized), outfile)
    return (ifile, pfile)



import logging
import argparse
import operator

import ruamel
from ruamel.yaml import YAML

yaml = YAML(typ="rt")
yaml.default_flow_style = False
yaml.allow_unicode = True


def load_yaml(foo):
    try:
        with open(foo, "r") as file:
            return yaml.load(file)
    except ruamel.yaml.constructor.DuplicateKeyError as msg:
        logger = logging.getLogger(__name__)
        error = "\n".join(str(msg).split("\n")[2:-7])
        logger.error(error)
        raise SystemExit


class ColoredFormatter(logging.Formatter):
    """                                                                         
        Class to create a log output which is colored based on level.           
    """

    def __init__(self, *args, **kwargs):
        super(ColoredFormatter, self).__init__(*args, **kwargs)
        self.colors = {
            'DEBUG': '\033[94m',
            'INFO': '\033[92m',
            'WARNING': '\033[93m',
            'ERROR': '\033[91m',
        }

        self.reset = '\033[0m'

    def format(self, record):
        msg = str(record.msg)
        level_name = str(record.levelname)
        name = str(record.name)
        color_prefix = self.colors[level_name]
        return '{0}{1:<9s} : {2}{3}'.format(color_prefix,
                                            '[' + level_name + ']', msg,
                                            self.reset)


def setup_logging(log_level):
    """Setup logging                                                            
                                                                                
        Verbosity decided on user input                                         
                                                                                                                                                   
        :param log_level: User defined log level                             
                                                                                
        :type log_level: str                                                               
    """
    numeric_level = getattr(logging, log_level.upper(), None)

    if not isinstance(numeric_level, int):
        print(
            "\033[91mInvalid log level passed. Please select from debug | info | warning | error\033[0m"
        )
        raise ValueError("{}-Invalid log level.".format(log_level))

    logging.basicConfig(level=numeric_level)


class SortingHelpFormatter(argparse.HelpFormatter):

    def add_arguments(self, actions):
        actions = sorted(actions, key=operator.attrgetter('option_strings'))
        super(SortingHelpFormatter, self).add_arguments(actions)


def rifle_cmdline_args():
    parser = argparse.ArgumentParser(formatter_class=SortingHelpFormatter,
                                     prog="rifle",
                                     description="RISC-V Feature Legalizer")
    parser.add_argument('--isa_spec',
                        '-ispec',
                        type=str,
                        metavar='YAML',
                        help='The YAML which contains the ISA specs.',
                        required=True)
    parser.add_argument('--platform_spec',
                        '-pspec',
                        type=str,
                        metavar='YAML',
                        help='The YAML which contains the Platfrorm specs.',
                        required=True),
    parser.add_argument(
        '--work_dir',
        type=str,
        default="rifle_work",
        metavar='DIR',
        help='The name of the work dir to dump the output files to.'),
    parser.add_argument('--verbose',
                        action='store',
                        default='info',
                        help='debug | info | warning | error',
                        metavar="")
    return parser



import os

root = os.path.abspath(os.path.dirname(__file__))

isa_schema = os.path.join(root, 'schemas/schema_isa.yaml')
platform_schema = os.path.join(root, 'schemas/schema_platform.yaml')

priv_versions = ["1.10", "1.11"]
user_versions = ["2.2", "2.3"]



import os
import codecs
import pip
from setuptools import setup, find_packages

import rifle

# Base directory of package
here = os.path.abspath(os.path.dirname(__file__))


def read(*parts):
    with codecs.open(os.path.join(here, *parts), 'r') as fp:
        return fp.read()


#Long Description
with open("README.rst", "r") as fh:
    long_description = fh.read()

setup(name="rifle",
      version=rifle.__version__,
      description="RISC-V Featrue Legalizer by Incoresemi Ltd.",
      long_description=long_description,
      classifiers=[
          "Programming Language :: Python :: 3.7",
          "License :: OSI Approved :: BSD License",
          "Development Status :: 4 - Beta"
      ],
      url='https://gitlab.com/incoresemi/rifle',
      author='InCore Semiconductors Pvt. Ltd.',
      author_email='neelgala@incoresemi.com',
      license='BSD-3-Clause',
      packages=find_packages(),
      install_package_data=True,
      package_dir={'rifle': 'rifle/'},
      package_data={'rifle': ['schemas/*']},
      install_requires=['Cerberus>=1.3.1', 'ruamel.yaml>=0.16.0'],
      python_requires=">=3.7.0",
      entry_points={
          "console_scripts": ["rifle=rifle.main:main"],
      })



from cerberus import Validator
import rifle.constants as constants


class schemaValidator(Validator):
    ''' Custom validator for schema having the custom rules necessary for implementation and checks.'''

    def __init__(self, *args, **kwargs):
        global xlen
        global extensions
        xlen = kwargs.get('xlen')
        super(schemaValidator, self).__init__(*args, **kwargs)

    def _check_with_priv_version_check(self, field, value):
        '''Function to check whether the Privileged spec version specified is valid or not.'''
        if value not in constants.priv_versions:
            self._error(
                field,
                "Invalid privilege spec version. Please select one of the following- "
                + ", ".join(constants.priv_versions))

    def _check_with_user_version_check(self, field, value):
        '''Function to check whether the User spec version specified is valid or not.'''
        if value not in constants.user_versions:
            self._error(
                field,
                "Invalid User spec version. Please select one of the following- "
                + ", ".join(constants.user_versions))

    def _check_with_capture_isa_specifics(self, field, value):
        '''
        Function to extract and store ISA specific information(such as xlen,user 
        spec version and extensions present)
        and check whether the dependencies in ISA extensions are satisfied.
        '''
        global xlen
        global extensions
        extension_enc = list("00000000000000000000000000")
        if "32" in value:
            xlen = 32
            ext = value[4:]
        elif "64" in value:
            xlen = 64
            ext = value[4:]
        elif "128" in value:
            xlen = 128
            ext = value[5:]
        else:
            self._error(field, "Invalid width in ISA.")
        #ISA checks
        if any(x in value for x in "EI"):
            if 'D' in value and not 'F' in value:
                self._error(field, "D cannot exist without F.")
            if 'Q' in value and not all(x in value for x in "FD"):
                self._error(field, "Q cannot exist without F and D.")
            if 'F' in value and not "Zicsr" in value:
                self._error(field, "F cannot exist without Zicsr.")
            if 'Zam' in value and not 'A' in value:
                self._error(field, "Zam cannot exist without A.")
            if 'N' in value and not 'U' in value:
                self._error(field, "N cannot exist without U.")
            if 'S' in value and not 'U' in value:
                self._error(field, "S cannot exist without U.")
            if 'Z' in value and not self.document['User_Spec_Version'] == "2.3":
                self._error(field, "Z is not supported in the given version.")
        else:
            self._error(field, "Neither of E or I extensions are present.")
        #ISA encoding for future use.
        for x in "ACDEFGIJLMNPQSTUVXZ":
            if (x in ext):
                extension_enc[25 - int(ord(x) - ord('A'))] = "1"
        extensions = int("".join(extension_enc), 2)

    def _check_with_max_length(self, field, value):
        '''Function to check whether the given value is less than the maximum value that can be stored(2^xlen-1).'''
        global xlen
        global extensions
        if value > (2**xlen) - 1:
            self._error(field, "Max value is greater than " + str(2**xlen - 1))

    def _check_with_len_check(self, field, value):
        '''Function to check whether the given value is less than XLEN/32(For check).'''
        global xlen
        global extensions
        maxv = xlen / 32 + 1
        for list in value:
            if (len(list) > 2):
                self._error(field,
                            "Only two values are allowed in each sub list.")
            for val in list:
                if not (val < maxv):
                    self._error(field, "Invalid values.")

    def _check_with_sxl_check(self, field, value):
        '''Function to check whether the input list for SXL field is valid.'''
        global extensions
        global xlen
        global sxlen
        maxv = xlen / 32
        allowed = []
        for list in value:
            if (len(list) > 2):
                self._error(field,
                            "Only two values are allowed in each sub list.")
            if (len(list) == 2):
                for x in range(list[0], list[1] + 1):
                    allowed.append(x)
            else:
                allowed.append(list[0])
        sxlen = max(allowed)
        if any(x > maxv for x in allowed):
            self._error(field, "Max allowed value is " + str(maxv))
        if 0 in allowed:
            if len(allowed) > 1:
                self._error(
                    field,
                    "0 is not allowed as a legal value with other values")
            elif extensions & int("0040000", 16) == 0:
                self._error(
                    field,
                    "SXL cannot be hardwired to 0 when S mode is supported")

    def _check_with_uxl_check(self, field, value):
        '''Function to check whether the input list for UXL field is valid.'''
        global extensions
        global xlen
        maxv = xlen / 32
        allowed = []
        for list in value:
            if (len(list) > 2):
                self._error(field,
                            "Only two values are allowed in each sub list.")
            if (len(list) == 2):
                for x in range(list[0], list[1] + 1):
                    allowed.append(x)
            else:
                allowed.append(list[0])
        if any(x > maxv for x in allowed):
            self._error(field, "Max allowed value is " + str(maxv))
        if 0 in allowed:
            if len(allowed) > 1:
                self._error(
                    field,
                    "0 is not allowed as a legal value with other values")
            elif extensions & int("0100000", 16) == 0:
                self._error(
                    field,
                    "UXL cannot be hardwired to 0 when S mode is supported")

    def _check_with_ext_check(self, field, value):
        '''Function to check whether the bitmask given for the Extensions field in misa is valid.'''
        global xlen
        global extensions
        val = value['mask'] ^ value['default'] ^ extensions
        if (val > 0):
            self._error(field, "Extension Bitmask error.")

    def _check_with_mpp_check(self, field, value):
        '''Function to check whether the modes specified in MPP field in mstatus is supported'''
        global extensions
        allowed = []
        for list in value:
            if (len(list) > 2):
                self._error(field,
                            "Only two values are allowed in each sub list.")
            if (len(list) == 2):
                for x in range(list[0], list[1] + 1):
                    allowed.append(x)
            else:
                allowed.append(list[0])
        if (0 in allowed) and extensions & int("0100000", 16) == 0:
            self._error(field,
                        "0 not a valid entry as U extension is not supported.")
        if (1 in allowed) and extensions & int("0040000", 16) == 0:
            self._error(field,
                        "1 not a valid entry as S extension is not supported.")

    def _check_with_xtveccheck(self, field, value):
        '''Function to check whether the inputs in range type in mtvec are valid.'''
        global xlen
        maxv = 2**(xlen - 2)
        for list in value:
            if (len(list) > 2):
                self._error(field,
                            "Only two values are allowed in each sub list.")
            for val in list:
                if not (val < maxv):
                    self._error(field, "Invalid values.")

    def _check_with_hardwirecheck(self, field, value):
        '''Function to check that none of the bits in the field are hardwired to 1'''
        if (value['bitmask']['default'] > 0):
            self._error(field, "No bit can be harwired to 1.")

    def _check_with_medelegcheck(self, field, value):
        '''Function to check that the input given for medeleg satisfies the constraints'''
        if (value['bitmask']['mask'] & int("800", 16) > 0):
            self._error(field, "11th bit must be hardwired to 0.")
        if (value['bitmask']['default'] > 0):
            self._error(field, "No bit can be harwired to 1.")

    def _check_with_rangecheck(self, field, value):
        '''Function to check whether the inputs in range type in WARL fields are valid.'''
        global xlen
        maxv = 2**(xlen)
        for list in value:
            if (len(list) > 2):
                self._error(field,
                            "Only two values are allowed in each sub list.")
            for val in list:
                if not (val < maxv):
                    self._error(field, "Value greater than " + str(maxv))

    def _check_with_xcause_check(self, field, value):
        '''Function to verify the inputs for mcause.'''
        if (min(value) < 16):
            self._error(
                field, "Invalid platform specific values for exception cause.")

    def _check_with_satp_check(self, field, value):
        global sxlen
        if value['implemented']:
            try:
                if sxlen == 1:
                    for list in value['MODE']['range']['rangelist']:
                        if (len(list) > 2):
                            self._error(
                                field,
                                "Only two values are allowed in each sub list.")
                        for val in list:
                            if not val in [0, 1]:
                                self._error(field, "Invalid value " + str(val))
                    asidlen = 9
                    ppnlen = 22
                elif sxlen == 2:
                    allowed = []
                    for list in value['MODE']['range']['rangelist']:
                        if (len(list) > 2):
                            self._error(
                                field,
                                "Only two values are allowed in each sub list.")
                        for val in list:
                            if not val in [0, 8, 9]:
                                self._error(field, "Invalid value " + str(val))
                            allowed.append(val)
                    if 9 in allowed and 8 not in allowed:
                        self._error(field, "9 cannot be present without 8.")
                    asidlen = 16
                    ppnlen = 44
                maxv = (2**asidlen) - 1
                if 'bitmask' in value['ASID']:
                    if value['ASID']['bitmask']['mask'] > maxv:
                        self._error(
                            field, "ASID-Bitmask-mask value is greater than " +
                            str(maxv))
                    if value['ASID']['bitmask']['mask'] > maxv:
                        self._error(
                            field,
                            "ASID-Bitmask-default value is greater than " +
                            str(maxv))
                if 'range' in value['ASID']:
                    for list in value['ASID']['range']['rangelist']:
                        if (len(list) > 2):
                            self._error(
                                field,
                                "Only two values are allowed in each sub list.")
                        for val in list:
                            if val > maxv:
                                self._error(
                                    field,
                                    "ASID-range-rangelist Value greater than " +
                                    str(maxv))
                maxv = (2**ppnlen) - 1
                if 'bitmask' in value['PPN']:
                    if value['PPN']['bitmask']['mask'] > maxv:
                        self._error(
                            field, "PPN-Bitmask-mask value is greater than " +
                            str(maxv))
                    if value['PPN']['bitmask']['mask'] > maxv:
                        self._error(
                            field,
                            "PPN-Bitmask-default value is greater than " +
                            str(maxv))
                if 'range' in value['PPN']:
                    for list in value['PPN']['range']['rangelist']:
                        if (len(list) > 2):
                            self._error(
                                field,
                                "Only two values are allowed in each sub list.")
                        for val in list:
                            if val > maxv:
                                self._error(
                                    field,
                                    "PPN-range-rangelist Value greater than " +
                                    str(maxv))
            except KeyError:
                self._error(
                    field,
                    "Please refer to the schema and enter the node values correctly."
                )
